import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import onClickOutside from 'react-onclickoutside';
import invariant from 'invariant';
import moment from 'moment';

import * as lang from '../../utils/lang';
import FoundationTether from '../misc/FoundationTether';
import isNullOrEmpty from '../../utils/isNullOrEmpty';
import isHandheld from '../../utils/isHandheld';
import toTimestamp from '../../utils/toTimestamp';

import DatepickerCalendar from './DatepickerCalendar';
import DatepickerControl from './DatepickerControl';
import isVisibleInCurrentMonth from './utils/isVisibleInCurrentMonth';


export default ({
    block,
    displayName,
    uiPropDefaultType = PropTypes.string,
    uiPropDefaultValue = '',
    uiPropGetValue = (ui) => ui
}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    // These are extracted for better testability because I prefer to select based on
    // display name instead of CSS class.
    const Calendar = DatepickerCalendar({ block, displayName: `${displayName}Calendar` });
    const Control = DatepickerControl({ block, displayName: `${displayName}Control` });

    return onClickOutside(React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            currentMonth: PropTypes.number,
            disabled: PropTypes.bool,
            emptyText: PropTypes.string,
            expanded: PropTypes.bool,
            focus: PropTypes.bool,
            hasGrayedOutPast: PropTypes.bool,
            onChange: PropTypes.func,
            onCollapse: PropTypes.func,
            onExpand: PropTypes.func,
            onSelectedIndexChange: PropTypes.func,
            selectedIndex: PropTypes.number,
            tetherTo: PropTypes.string,
            ui: uiPropDefaultType,
            value: PropTypes.oneOfType([
                PropTypes.instanceOf(Date),
                PropTypes.number
            ])
        },

        getDefaultProps() {
            const timestamp = +new Date();
            return {
                currentMonth: timestamp,
                disabled: false,
                emptyText: lang.pgettext('foundation-datepicker', 'Label--no-date-yet'),
                expanded: false,
                focus: false,
                hasGrayedOutPast: false,
                onChange: (/* currentMonth, selectedIndex, value */) => {},
                onCollapse: (/* currentMonth, selectedIndex */) => {},
                onExpand: () => {},
                onSelectedIndexChange: (/* currentMonth, selectedIndex */) => {},
                selectedIndex: timestamp,
                tetherTo: 'content',
                ui: uiPropDefaultValue,
                value: null
            };
        },

        componentDidMount() {
            if (this.props.focus) {
                ReactDOM.findDOMNode(this.component).focus();
            }
        },

        handleClickOutside() {
            if (this.props.expanded) {
                this.collapse();
            }
        },

        onCollapse() {
            this.collapse();
        },

        onKeyDown(evt) {
            let date, newCurrentMonth;
            let currentMonth = moment(this.props.currentMonth);

            switch (evt.which) {
                case 9: // tab
                    if (this.props.expanded) {
                        // don't tab to next form control when expanded.
                        evt.stopPropagation();
                        evt.preventDefault();

                        if (evt.shiftKey) {
                            this.moveToPreviousMonth();
                        } else {
                            this.moveToNextMonth();
                        }
                    }

                    break;
                case 13: // enter
                case 32: // space
                    evt.stopPropagation();
                    evt.preventDefault();

                    if (this.props.expanded) {
                        this.change(moment(this.props.selectedIndex));
                    } else {
                        this.expand();
                    }

                    break;
                case 27: // escape
                    if (this.props.expanded) {
                        this.collapse();
                    }

                    break;
                case 37: // left
                    evt.preventDefault();

                    if (this.props.expanded) {
                        date = moment(this.props.selectedIndex).subtract(1, 'days');
                        newCurrentMonth = isVisibleInCurrentMonth(currentMonth, date)
                            ? currentMonth
                            : currentMonth.clone().subtract(1, 'months');

                        this.props.onSelectedIndexChange(toTimestamp(newCurrentMonth), toTimestamp(date));
                    }

                    break;
                case 38: // up
                    evt.preventDefault();

                    if (this.props.expanded) {
                        date = moment(this.props.selectedIndex).subtract(7, 'days');
                        newCurrentMonth = isVisibleInCurrentMonth(currentMonth, date)
                            ? currentMonth
                            : currentMonth.clone().subtract(1, 'months');

                        this.props.onSelectedIndexChange(toTimestamp(newCurrentMonth), toTimestamp(date));
                    }

                    break;
                case 39: // right
                    evt.preventDefault();

                    if (this.props.expanded) {
                        date = moment(this.props.selectedIndex).add(1, 'days');
                        newCurrentMonth = isVisibleInCurrentMonth(currentMonth, date)
                            ? currentMonth
                            : currentMonth.clone().add(1, 'months');

                        this.props.onSelectedIndexChange(toTimestamp(newCurrentMonth), toTimestamp(date));
                    }

                    break;
                case 40: // down
                    evt.preventDefault();

                    if (this.props.expanded) {
                        date = moment(this.props.selectedIndex).add(7, 'days');
                        newCurrentMonth = isVisibleInCurrentMonth(currentMonth, date)
                            ? currentMonth
                            : currentMonth.clone().add(1, 'months');

                        this.props.onSelectedIndexChange(toTimestamp(newCurrentMonth), toTimestamp(date));
                    } else {
                        this.expand();
                    }

                    break;
            }
        },

        onNextMonthClick() {
            this.moveToNextMonth();
        },

        onExpand() {
            this.expand();
        },

        onPreviousMonthClick() {
            this.moveToPreviousMonth();
        },

        onToggle() {
            const { disabled, expanded } = this.props;
            if (disabled) {
                return;
            }

            expanded ? this.collapse() : this.expand();
        },

        onTouchableChange(evt) {
            if (evt.target.value === '') {
                this.change(null);
            } else {
                const date = moment(evt.target.value, 'YYYY-MM-DD');
                this.change(date);
            }
        },

        change(date) {
            const { disabled, onChange } = this.props;
            if (disabled) {
                return;
            }

            if (date === null) {
                const now = moment();
                onChange(toTimestamp(now.clone().startOf('month')), toTimestamp(now.clone()), null);
            } else if (date.isValid()) {
                onChange(toTimestamp(date.clone().startOf('month')), toTimestamp(date.clone()), toTimestamp(date.clone()));
            }
        },

        collapse() {
            if (isHandheld()) {
                return;
            }

            const currentMonth = this.props.value !== null
                ? toTimestamp(moment(this.props.value).startOf('month'))
                : +new Date();

            const selectedIndex = this.props.value !== null
                ? toTimestamp(moment(this.props.value))
                : +new Date();

            this.props.onCollapse(currentMonth, selectedIndex);
        },

        expand() {
            if (isHandheld()) {
                return;
            }

            this.props.onExpand();
        },

        moveToNextMonth() {
            const { currentMonth, onSelectedIndexChange, selectedIndex } = this.props;
            onSelectedIndexChange(
                toTimestamp(moment(currentMonth).add(1, 'months')),
                toTimestamp(moment(selectedIndex).add(1, 'months'))
            );
        },

        moveToPreviousMonth() {
            const { currentMonth, onSelectedIndexChange, selectedIndex } = this.props;
            onSelectedIndexChange(
                toTimestamp(moment(currentMonth).subtract(1, 'months')),
                toTimestamp(moment(selectedIndex).subtract(1, 'months'))
            );
        },

        renderCalendar() {
            const { currentMonth, hasGrayedOutPast, expanded, onChange, selectedIndex, tetherTo, value } = this.props;
            if (!expanded || isHandheld()) {
                return null;
            }

            return (
                <FoundationTether
                    target={ () => ReactDOM.findDOMNode(this) }
                    tetherTo={ tetherTo }>
                    <Calendar
                        currentMonth={ moment(currentMonth) }
                        hasGrayedOutPast={ hasGrayedOutPast }
                        onNextMonthClick={ this.onNextMonthClick }
                        onPreviousMonthClick={ this.onPreviousMonthClick }
                        onChange={ onChange }
                        selectedIndex={ moment(selectedIndex) }
                        value={ moment(value) } />
                </FoundationTether>
            );
        },

        render() {
            const { disabled, expanded, ui } = this.props;
            const touchable = isHandheld();

            const value = this.props.value !== null
                ? moment(this.props.value).format('YYYY-MM-DD')
                : this.props.emptyText;

            return (
                <div
                    className={ block({ disabled, expanded, touchable }, uiPropGetValue(ui)) }
                    onKeyDown={ this.onKeyDown }
                    ref={ (c) => this.component = c }
                    tabIndex={ disabled ? null : '0' }>
                    <Control
                        defaultValue={ this.props.value !== null ? value : null }
                        disabled={ disabled }
                        onClick={ this.onToggle }
                        onTouchableBlur={ this.onCollapse }
                        onTouchableChange={ this.onTouchableChange }
                        onTouchableFocus={ this.onExpand }
                        value={ value } />
                    { this.renderCalendar() }
                </div>
            );
        }

    }));

};
